# coding=utf-8
from __future__ import division

from ladybug import datatype
from ladybug.datatype import base
from ladybug.datatype import angle, area, distance, energy, energyflux, \
    energyintensity, fraction, generic, illuminance, luminance, mass, massflowrate, \
    power, pressure, rvalue, speed, temperature, temperaturedelta, temperaturetime, \
    thermalcondition, specificenergy, uvalue, volume, volumeflowrate

import unittest
import pytest
import math
PI = math.pi


class DataTypesTestCase(unittest.TestCase):
    """Test for (ladybug/datatype.py)"""

    # preparing to test.
    def setUp(self):
        """set up."""

    def tearDown(self):
        """Nothing to tear down as nothing gets written to file."""
        pass

    def test_global_properties(self):
        """Test the global properties like TYPES, BASETYPES, and UNITS."""
        assert isinstance(datatype.TYPES, tuple)
        assert isinstance(datatype.BASETYPES, tuple)
        assert isinstance(datatype.UNITS, dict)
        assert isinstance(datatype.TYPESDICT, dict)

    def test_from_json(self):
        """Test the from json method."""
        sample_json = {'name': 'Temperature',
                       'data_type': 'Temperature',
                       'base_unit': 'C'}
        new_temp = base.DataTypeBase.from_json(sample_json)
        assert isinstance(new_temp, temperature.Temperature)

    def test_to_from_json(self):
        """Test the to/from json methods."""
        my_temp = temperature.Temperature()
        temp_json = my_temp.to_json()
        new_temp = base.DataTypeBase.from_json(temp_json)
        new_temp = base.DataTypeBase.from_json(temp_json)
        assert new_temp.to_json() == temp_json

    def test_json_generic(self):
        """Test the from json for a generic type."""
        sample_json = {'name': 'Days Since Last Snowfall',
                       'data_type': 'GenericType',
                       'base_unit': 'days'}
        new_type = base.DataTypeBase.from_json(sample_json)
        assert isinstance(new_type, base.DataTypeBase)

    def test_generic_type(self):
        """Test the creation of generic types."""
        test_type = generic.GenericType('Test Type', 'widgets')
        assert hasattr(test_type, 'isDataType')
        assert test_type.is_unit_acceptable('widgets')

    def test_temperature(self):
        """Test Temperature type."""
        temp_type = temperature.Temperature()
        for unit in temp_type.units:
            assert temp_type.to_unit([1], unit, unit)[0] == pytest.approx(1, rel=1e-5)
            ip_vals, ip_u = temp_type.to_ip([1], unit)
            assert len(ip_vals) == 1
            si_vals, si_u = temp_type.to_si([1], unit)
            assert len(si_vals) == 1
            for other_unit in temp_type.units:
                assert len(temp_type.to_unit([1], other_unit, unit)) == 1
        assert temp_type.to_unit([1], 'F', 'C')[0] == pytest.approx(33.8, rel=1e-1)
        assert temp_type.to_unit([1], 'K', 'C')[0] == pytest.approx(274.15, rel=1e-1)
        assert temp_type.to_unit([1], 'C', 'F')[0] == pytest.approx(-17.2222, rel=1e-1)
        assert temp_type.to_unit([1], 'C', 'K')[0] == pytest.approx(-272.15, rel=1e-1)

    def test_temperaturedelta(self):
        """Test TemperatureDelta type."""
        temp_type = temperaturedelta.TemperatureDelta()
        for unit in temp_type.units:
            assert temp_type.to_unit([1], unit, unit)[0] == pytest.approx(1, rel=1e-5)
            ip_vals, ip_u = temp_type.to_ip([1], unit)
            assert len(ip_vals) == 1
            si_vals, si_u = temp_type.to_si([1], unit)
            assert len(si_vals) == 1
            for other_unit in temp_type.units:
                assert len(temp_type.to_unit([1], other_unit, unit)) == 1
        assert temp_type.to_unit([1], 'F', 'C')[0] == pytest.approx(1.8, rel=1e-1)
        assert temp_type.to_unit([1], 'K', 'C')[0] == pytest.approx(1, rel=1e-1)
        assert temp_type.to_unit([1], 'C', 'F')[0] == pytest.approx(0.5555, rel=1e-1)
        assert temp_type.to_unit([1], 'C', 'K')[0] == pytest.approx(1, rel=1e-1)

    def test_temperaturetime(self):
        """Test TemperatureTime type."""
        temp_type = temperaturetime.TemperatureTime()
        for unit in temp_type.units:
            assert temp_type.to_unit([1], unit, unit)[0] == pytest.approx(1, rel=1e-5)
            ip_vals, ip_u = temp_type.to_ip([1], unit)
            assert len(ip_vals) == 1
            si_vals, si_u = temp_type.to_si([1], unit)
            assert len(si_vals) == 1
            for other_unit in temp_type.units:
                assert len(temp_type.to_unit([1], other_unit, unit)) == 1
        assert temp_type.to_unit([1], 'degF-days', 'degC-days')[0] == pytest.approx(1.8, rel=1e-1)
        assert temp_type.to_unit([1], 'degC-hours', 'degC-days')[0] == pytest.approx(24, rel=1e-3)
        assert temp_type.to_unit([1], 'degF-hours', 'degC-days')[0] == pytest.approx(43.2, rel=1e-1)
        assert temp_type.to_unit([1], 'degC-days', 'degF-days')[0] == pytest.approx(0.5555, rel=1e-1)
        assert temp_type.to_unit([1], 'degC-days', 'degC-hours')[0] == pytest.approx(1 / 24, rel=1e-1)
        assert temp_type.to_unit([1], 'degC-days', 'degF-hours')[0] == pytest.approx(0.023148, rel=1e-1)

    def test_fraction(self):
        """Test Fraction type."""
        pct_type = fraction.Fraction()
        for unit in pct_type.units:
            assert pct_type.to_unit([1], unit, unit)[0] == pytest.approx(1, rel=1e-5)
            ip_vals, ip_u = pct_type.to_ip([1], unit)
            assert len(ip_vals) == 1
            si_vals, si_u = pct_type.to_si([1], unit)
            assert len(si_vals) == 1
            for other_unit in pct_type.units:
                assert len(pct_type.to_unit([1], other_unit, unit)) == 1
        assert pct_type.to_unit([1], 'fraction', '%')[0] == 0.01
        assert pct_type.to_unit([1], 'tenths', '%')[0] == 0.1
        assert pct_type.to_unit([1], 'thousandths', '%')[0] == 10
        assert pct_type.to_unit([1], '%', 'fraction')[0] == 100
        assert pct_type.to_unit([1], '%', 'tenths')[0] == 10
        assert pct_type.to_unit([1], '%', 'thousandths')[0] == 0.1

    def test_distance(self):
        """Test Distance type."""
        dist_type = distance.Distance()
        for unit in dist_type.units:
            assert dist_type.to_unit([1], unit, unit)[0] == pytest.approx(1, rel=1e-5)
            ip_vals, ip_u = dist_type.to_ip([1], unit)
            assert len(ip_vals) == 1
            si_vals, si_u = dist_type.to_si([1], unit)
            assert len(si_vals) == 1
            for other_unit in dist_type.units:
                assert len(dist_type.to_unit([1], other_unit, unit)) == 1
        assert dist_type.to_unit([1], 'ft', 'm')[0] == pytest.approx(3.28084, rel=1e-2)
        assert dist_type.to_unit([1], 'mm', 'm')[0] == 1000
        assert dist_type.to_unit([1], 'in', 'm')[0] == pytest.approx(39.3701, rel=1e-1)
        assert dist_type.to_unit([1], 'km', 'm')[0] == 0.001
        assert dist_type.to_unit([1], 'mi', 'm')[0] == pytest.approx(1 / 1609.344, rel=1e-1)
        assert dist_type.to_unit([1], 'cm', 'm')[0] == 100
        assert dist_type.to_unit([1], 'm', 'ft')[0] == pytest.approx(1 / 3.28084, rel=1e-2)
        assert dist_type.to_unit([1], 'm', 'mm')[0] == 0.001
        assert dist_type.to_unit([1], 'm', 'in')[0] == pytest.approx(1 / 39.3701, rel=1e-1)
        assert dist_type.to_unit([1], 'm', 'km')[0] == 1000
        assert dist_type.to_unit([1], 'm', 'mi')[0] == pytest.approx(1609.344, rel=1e-1)
        assert dist_type.to_unit([1], 'm', 'cm')[0] == 0.01

    def test_area(self):
        """Test Area type."""
        area_type = area.Area()
        for unit in area_type.units:
            assert area_type.to_unit([1], unit, unit)[0] == pytest.approx(1, rel=1e-5)
            ip_vals, ip_u = area_type.to_ip([1], unit)
            assert len(ip_vals) == 1
            si_vals, si_u = area_type.to_si([1], unit)
            assert len(si_vals) == 1
            for other_unit in area_type.units:
                assert len(area_type.to_unit([1], other_unit, unit)) == 1
        assert area_type.to_unit([1], 'ft2', 'm2')[0] == pytest.approx(10.7639, rel=1e-2)
        assert area_type.to_unit([1], 'mm2', 'm2')[0] == 1000000
        assert area_type.to_unit([1], 'in2', 'm2')[0] == pytest.approx(1550, rel=1e-1)
        assert area_type.to_unit([1], 'km2', 'm2')[0] == 0.000001
        assert area_type.to_unit([1], 'mi2', 'm2')[0] == pytest.approx(1 / 2590000, rel=1e-8)
        assert area_type.to_unit([1], 'cm2', 'm2')[0] == 10000
        assert area_type.to_unit([1], 'ha', 'm2')[0] == 0.0001
        assert area_type.to_unit([1], 'acre', 'm2')[0] == pytest.approx(1 / 4046.86, rel=1e-8)
        assert area_type.to_unit([1], 'm2', 'ft2')[0] == pytest.approx(1 / 10.7639, rel=1e-3)
        assert area_type.to_unit([1], 'm2', 'mm2')[0] == 0.000001
        assert area_type.to_unit([1], 'm2', 'in2')[0] == pytest.approx(1 / 1550, rel=1e-4)
        assert area_type.to_unit([1], 'm2', 'km2')[0] == 1000000
        assert area_type.to_unit([1], 'm2', 'mi2')[0] == pytest.approx(2590000, rel=1e-1)
        assert area_type.to_unit([1], 'm2', 'cm2')[0] == 0.0001
        assert area_type.to_unit([1], 'm2', 'ha')[0] == 10000
        assert area_type.to_unit([1], 'm2', 'acre')[0] == pytest.approx(4046.86, rel=1e-1)

    def test_volume(self):
        """Test Volume type."""
        vol_type = volume.Volume()
        for unit in vol_type.units:
            assert vol_type.to_unit([1], unit, unit)[0] == pytest.approx(1, rel=1e-5)
            ip_vals, ip_u = vol_type.to_ip([1], unit)
            assert len(ip_vals) == 1
            si_vals, si_u = vol_type.to_si([1], unit)
            assert len(si_vals) == 1
            for other_unit in vol_type.units:
                assert len(vol_type.to_unit([1], other_unit, unit)) == 1
        assert vol_type.to_unit([1], 'ft3', 'm3')[0] == pytest.approx(35.3147, rel=1e-2)
        assert vol_type.to_unit([1], 'mm3', 'm3')[0] == 1e+9
        assert vol_type.to_unit([1], 'in3', 'm3')[0] == pytest.approx(61023.7, rel=1e-1)
        assert vol_type.to_unit([1], 'km3', 'm3')[0] == 0.000000001
        assert vol_type.to_unit([1], 'mi3', 'm3')[0] == pytest.approx(1 / 4.168e+9, rel=1e-8)
        assert vol_type.to_unit([1], 'L', 'm3')[0] == 1000
        assert vol_type.to_unit([1], 'mL', 'm3')[0] == 1000000
        assert vol_type.to_unit([1], 'gal', 'm3')[0] == pytest.approx(264.172, rel=1e-1)
        assert vol_type.to_unit([1], 'fl oz', 'm3')[0] == pytest.approx(33814, rel=1e-1)
        assert vol_type.to_unit([1], 'm3', 'ft3')[0] == pytest.approx(1 / 35.3147, rel=1e-3)
        assert vol_type.to_unit([1], 'm3', 'mm3')[0] == 0.000000001
        assert vol_type.to_unit([1], 'm3', 'in3')[0] == pytest.approx(1 / 61023.7, rel=1e-5)
        assert vol_type.to_unit([1], 'm3', 'km3')[0] == 1e+9
        assert vol_type.to_unit([1], 'm3', 'mi3')[0] == pytest.approx(4.168e+9, rel=1e-1)
        assert vol_type.to_unit([1], 'm3', 'L')[0] == 0.001
        assert vol_type.to_unit([1], 'm3', 'mL')[0] == 0.000001
        assert vol_type.to_unit([1], 'm3', 'gal')[0] == pytest.approx(1 / 264.172, rel=1e-4)
        assert vol_type.to_unit([1], 'm3', 'fl oz')[0] == pytest.approx(1 / 33814, rel=1e-5)

    def test_pressure(self):
        """Test Pressure type."""
        press_type = pressure.Pressure()
        for unit in press_type.units:
            assert press_type.to_unit([1], unit, unit)[0] == pytest.approx(1, rel=1e-5)
            ip_vals, ip_u = press_type.to_ip([1], unit)
            assert len(ip_vals) == 1
            si_vals, si_u = press_type.to_si([1], unit)
            assert len(si_vals) == 1
            for other_unit in press_type.units:
                assert len(press_type.to_unit([1], other_unit, unit)) == 1
        assert press_type.to_unit([1], 'inHg', 'Pa')[0] == pytest.approx(0.0002953, rel=1e-5)
        assert press_type.to_unit([1], 'atm', 'Pa')[0] == pytest.approx(1 / 101325, rel=1e-7)
        assert press_type.to_unit([1], 'bar', 'Pa')[0] == 0.00001
        assert press_type.to_unit([1], 'Torr', 'Pa')[0] == pytest.approx(0.00750062, rel=1e-5)
        assert press_type.to_unit([1], 'psi', 'Pa')[0] == pytest.approx(0.000145038, rel=1e-5)
        assert press_type.to_unit([1], 'inH2O', 'Pa')[0] == pytest.approx(0.00401865, rel=1e-5)
        assert press_type.to_unit([1], 'Pa', 'inHg')[0] == pytest.approx(1 / 0.0002953, rel=1e-1)
        assert press_type.to_unit([1], 'Pa', 'atm')[0] == pytest.approx(101325, rel=1e-1)
        assert press_type.to_unit([1], 'Pa', 'bar')[0] == 100000
        assert press_type.to_unit([1], 'Pa', 'Torr')[0] == pytest.approx(1 / 0.00750062, rel=1e-1)
        assert press_type.to_unit([1], 'Pa', 'psi')[0] == pytest.approx(1 / 0.000145038, rel=1e-1)
        assert press_type.to_unit([1], 'Pa', 'inH2O')[0] == pytest.approx(1 / 0.00401865, rel=1e-1)

    def test_energy(self):
        """Test Energy type."""
        energy_type = energy.Energy()
        for unit in energy_type.units:
            assert energy_type.to_unit([1], unit, unit)[0] == pytest.approx(1, rel=1e-5)
            ip_vals, ip_u = energy_type.to_ip([1], unit)
            assert len(ip_vals) == 1
            si_vals, si_u = energy_type.to_si([1], unit)
            assert len(si_vals) == 1
            for other_unit in energy_type.units:
                assert len(energy_type.to_unit([1], other_unit, unit)) == 1
        assert energy_type.to_unit([1], 'kBtu', 'kWh')[0] == pytest.approx(3.41214, rel=1e-2)
        assert energy_type.to_unit([1], 'Wh', 'kWh')[0] == 1000
        assert energy_type.to_unit([1], 'Btu', 'kWh')[0] == pytest.approx(3412.14, rel=1e-1)
        assert energy_type.to_unit([1], 'MMBtu', 'kWh')[0] == pytest.approx(0.00341214, rel=1e-5)
        assert energy_type.to_unit([1], 'J', 'kWh')[0] == 3600000
        assert energy_type.to_unit([1], 'kJ', 'kWh')[0] == 3600
        assert energy_type.to_unit([1], 'MJ', 'kWh')[0] == 3.6
        assert energy_type.to_unit([1], 'GJ', 'kWh')[0] == 0.0036
        assert energy_type.to_unit([1], 'therm', 'kWh')[0] == pytest.approx(0.0341214, rel=1e-5)
        assert energy_type.to_unit([1], 'cal', 'kWh')[0] == pytest.approx(860421, rel=1e-1)
        assert energy_type.to_unit([1], 'kcal', 'kWh')[0] == pytest.approx(860.421, rel=1e-2)
        assert energy_type.to_unit([1], 'kWh', 'kBtu')[0] == pytest.approx(1 / 3.41214, rel=1e-2)
        assert energy_type.to_unit([1], 'kWh', 'Wh')[0] == 0.001
        assert energy_type.to_unit([1], 'kWh', 'Btu')[0] == pytest.approx(1 / 3412.14, rel=1e-5)
        assert energy_type.to_unit([1], 'kWh', 'MMBtu')[0] == pytest.approx(1 / 0.00341214, rel=1e-1)
        assert energy_type.to_unit([1], 'kWh', 'J')[0] == 1 / 3600000
        assert energy_type.to_unit([1], 'kWh', 'kJ')[0] == 1 / 3600
        assert energy_type.to_unit([1], 'kWh', 'MJ')[0] == 1 / 3.6
        assert energy_type.to_unit([1], 'kWh', 'GJ')[0] == 1 / 0.0036
        assert energy_type.to_unit([1], 'kWh', 'therm')[0] == pytest.approx(1 / 0.0341214, rel=1e-1)
        assert energy_type.to_unit([1], 'kWh', 'cal')[0] == pytest.approx(1 / 860421, rel=1e-7)
        assert energy_type.to_unit([1], 'kWh', 'kcal')[0] == pytest.approx(1 / 860.421, rel=1e-4)

    def test_energy_intensity(self):
        """Test Energy type."""
        energyi_type = energyintensity.EnergyIntensity()
        for unit in energyi_type.units:
            assert energyi_type.to_unit([1], unit, unit)[0] == pytest.approx(1, rel=1e-5)
            ip_vals, ip_u = energyi_type.to_ip([1], unit)
            assert len(ip_vals) == 1
            si_vals, si_u = energyi_type.to_si([1], unit)
            assert len(si_vals) == 1
            for other_unit in energyi_type.units:
                assert len(energyi_type.to_unit([1], other_unit, unit)) == 1
        assert energyi_type.to_unit([1], 'kBtu/ft2', 'kWh/m2')[0] == pytest.approx(0.316998, rel=1e-3)
        assert energyi_type.to_unit([1], 'Wh/m2', 'kWh/m2')[0] == 1000
        assert energyi_type.to_unit([1], 'Btu/ft2', 'kWh/m2')[0] == pytest.approx(316.998, rel=1e-1)
        assert energyi_type.to_unit([1], 'kWh/m2', 'kBtu/ft2')[0] == pytest.approx(1 / 0.316998, rel=1e-2)
        assert energyi_type.to_unit([1], 'kWh/m2', 'Wh/m2')[0] == 0.001
        assert energyi_type.to_unit([1], 'kWh/m2', 'Btu/ft2')[0] == pytest.approx(1 / 316.998, rel=1e-5)

    def test_power(self):
        """Test Power type."""
        power_type = power.Power()
        for unit in power_type.units:
            assert power_type.to_unit([1], unit, unit)[0] == pytest.approx(1, rel=1e-5)
            ip_vals, ip_u = power_type.to_ip([1], unit)
            assert len(ip_vals) == 1
            si_vals, si_u = power_type.to_si([1], unit)
            assert len(si_vals) == 1
            for other_unit in power_type.units:
                assert len(power_type.to_unit([1], other_unit, unit)) == 1
        assert power_type.to_unit([1], 'Btu/h', 'W')[0] == pytest.approx(3.41214, rel=1e-3)
        assert power_type.to_unit([1], 'kW', 'W')[0] == 0.001
        assert power_type.to_unit([1], 'kBtu/h', 'W')[0] == pytest.approx(0.00341214, rel=1e-5)
        assert power_type.to_unit([1], 'TR', 'W')[0] == pytest.approx(1 / 3516.85, rel=1e-5)
        assert power_type.to_unit([1], 'hp', 'W')[0] == pytest.approx(1 / 745.7, rel=1e-5)
        assert power_type.to_unit([1], 'W', 'Btu/h')[0] == pytest.approx(1 / 3.41214, rel=1e-2)
        assert power_type.to_unit([1], 'W', 'kW')[0] == 1000
        assert power_type.to_unit([1], 'W', 'kBtu/h')[0] == pytest.approx(1 / 0.00341214, rel=1e-1)
        assert power_type.to_unit([1], 'W', 'TR')[0] == pytest.approx(3516.85, rel=1e-1)
        assert power_type.to_unit([1], 'W', 'hp')[0] == pytest.approx(745.7, rel=1e-1)

    def test_energy_flux(self):
        """Test EnergyFlux type."""
        energyf_type = energyflux.EnergyFlux()
        for unit in energyf_type.units:
            assert energyf_type.to_unit([1], unit, unit)[0] == pytest.approx(1, rel=1e-5)
            ip_vals, ip_u = energyf_type.to_ip([1], unit)
            assert len(ip_vals) == 1
            si_vals, si_u = energyf_type.to_si([1], unit)
            assert len(si_vals) == 1
            for other_unit in energyf_type.units:
                assert len(energyf_type.to_unit([1], other_unit, unit)) == 1
        assert energyf_type.to_unit([1], 'Btu/h-ft2', 'W/m2')[0] == pytest.approx(1 / 3.15459075, rel=1e-2)
        assert energyf_type.to_unit([1], 'kW/m2', 'W/m2')[0] == 0.001
        assert energyf_type.to_unit([1], 'kBtu/h-ft2', 'W/m2')[0] == pytest.approx(1 / 3154.59075, rel=1e-5)
        assert energyf_type.to_unit([1], 'W/ft2', 'W/m2')[0] == pytest.approx(1 / 10.7639, rel=1e-4)
        assert energyf_type.to_unit([1], 'met', 'W/m2')[0] == pytest.approx(1 / 58.2, rel=1e-4)
        assert energyf_type.to_unit([1], 'W/m2', 'Btu/h-ft2')[0] == pytest.approx(3.15459075, rel=1e-2)
        assert energyf_type.to_unit([1], 'W/m2', 'kW/m2')[0] == 1000
        assert energyf_type.to_unit([1], 'W/m2', 'kBtu/h-ft2')[0] == pytest.approx(3154.59075, rel=1e-1)
        assert energyf_type.to_unit([1], 'W/m2', 'W/ft2')[0] == pytest.approx(10.7639, rel=1e-1)
        assert energyf_type.to_unit([1], 'W/m2', 'met')[0] == pytest.approx(58.2, rel=1e-1)

    def test_illuminance(self):
        """Test Illuminance type."""
        ill_type = illuminance.Illuminance()
        for unit in ill_type.units:
            assert ill_type.to_unit([1], unit, unit)[0] == pytest.approx(1, rel=1e-5)
            ip_vals, ip_u = ill_type.to_ip([1], unit)
            assert len(ip_vals) == 1
            si_vals, si_u = ill_type.to_si([1], unit)
            assert len(si_vals) == 1
            for other_unit in ill_type.units:
                assert len(ill_type.to_unit([1], other_unit, unit)) == 1
        assert ill_type.to_unit([1], 'fc', 'lux')[0] == pytest.approx(1 / 10.7639, rel=1e-3)
        assert ill_type.to_unit([1], 'lux', 'fc')[0] == pytest.approx(10.7639, rel=1e-1)

    def test_luminance(self):
        """Test Luminance type."""
        lum_type = luminance.Luminance()
        for unit in lum_type.units:
            assert lum_type.to_unit([1], unit, unit)[0] == pytest.approx(1, rel=1e-5)
            ip_vals, ip_u = lum_type.to_ip([1], unit)
            assert len(ip_vals) == 1
            si_vals, si_u = lum_type.to_si([1], unit)
            assert len(si_vals) == 1
            for other_unit in lum_type.units:
                assert len(lum_type.to_unit([1], other_unit, unit)) == 1
        assert lum_type.to_unit([1], 'cd/ft2', 'cd/m2')[0] == pytest.approx(1 / 10.7639, rel=1e-3)
        assert lum_type.to_unit([1], 'cd/m2', 'cd/ft2')[0] == pytest.approx(10.7639, rel=1e-1)

    def test_angle(self):
        """Test Angle type."""
        ang_type = angle.Angle()
        for unit in ang_type.units:
            assert ang_type.to_unit([1], unit, unit)[0] == pytest.approx(1, rel=1e-5)
            ip_vals, ip_u = ang_type.to_ip([1], unit)
            assert len(ip_vals) == 1
            si_vals, si_u = ang_type.to_si([1], unit)
            assert len(si_vals) == 1
            for other_unit in ang_type.units:
                assert len(ang_type.to_unit([1], other_unit, unit)) == 1
        assert ang_type.to_unit([1], 'radians', 'degrees')[0] == pytest.approx(PI / 180, rel=1e-3)
        assert ang_type.to_unit([1], 'degrees', 'radians')[0] == pytest.approx((1 / PI) * 180, rel=1e-1)

    def test_mass(self):
        """Test Mass type."""
        mass_type = mass.Mass()
        for unit in mass_type.units:
            assert mass_type.to_unit([1], unit, unit)[0] == pytest.approx(1, rel=1e-5)
            ip_vals, ip_u = mass_type.to_ip([1], unit)
            assert len(ip_vals) == 1
            si_vals, si_u = mass_type.to_si([1], unit)
            assert len(si_vals) == 1
            for other_unit in mass_type.units:
                assert len(mass_type.to_unit([1], other_unit, unit)) == 1
        assert mass_type.to_unit([1], 'lb', 'kg')[0] == pytest.approx(2.20462, rel=1e-3)
        assert mass_type.to_unit([1], 'g', 'kg')[0] == 1000
        assert mass_type.to_unit([1], 'tonne', 'kg')[0] == 0.001
        assert mass_type.to_unit([1], 'ton', 'kg')[0] == pytest.approx(1 / 907.185, rel=1e-5)
        assert mass_type.to_unit([1], 'oz', 'kg')[0] == pytest.approx(35.274, rel=1e-1)
        assert mass_type.to_unit([1], 'kg', 'lb')[0] == pytest.approx(1 / 2.20462, rel=1e-3)
        assert mass_type.to_unit([1], 'kg', 'g')[0] == 0.001
        assert mass_type.to_unit([1], 'kg', 'tonne')[0] == 1000
        assert mass_type.to_unit([1], 'kg', 'ton')[0] == pytest.approx(907.185, rel=1e-1)
        assert mass_type.to_unit([1], 'kg', 'oz')[0] == pytest.approx(1 / 35.274, rel=1e-3)

    def test_speed(self):
        """Test Speed type."""
        speed_type = speed.Speed()
        for unit in speed_type.units:
            assert speed_type.to_unit([1], unit, unit)[0] == pytest.approx(1, rel=1e-5)
            ip_vals, ip_u = speed_type.to_ip([1], unit)
            assert len(ip_vals) == 1
            si_vals, si_u = speed_type.to_si([1], unit)
            assert len(si_vals) == 1
            for other_unit in speed_type.units:
                assert len(speed_type.to_unit([1], other_unit, unit)) == 1
        assert speed_type.to_unit([1], 'mph', 'm/s')[0] == pytest.approx(2.23694, rel=1e-3)
        assert speed_type.to_unit([1], 'km/h', 'm/s')[0] == 3.6
        assert speed_type.to_unit([1], 'knot', 'm/s')[0] == pytest.approx(1.94384, rel=1e-3)
        assert speed_type.to_unit([1], 'ft/s', 'm/s')[0] == pytest.approx(3.28084, rel=1e-3)
        assert speed_type.to_unit([1], 'm/s', 'mph')[0] == pytest.approx(1 / 2.23694, rel=1e-3)
        assert speed_type.to_unit([1], 'm/s', 'km/h')[0] == 1 / 3.6
        assert speed_type.to_unit([1], 'm/s', 'knot')[0] == pytest.approx(1 / 1.94384, rel=1e-3)
        assert speed_type.to_unit([1], 'm/s', 'ft/s')[0] == pytest.approx(1 / 3.28084, rel=1e-3)

    def test_volume_flow_rate(self):
        """Test VolumeFlowRate type."""
        vfr_type = volumeflowrate.VolumeFlowRate()
        for unit in vfr_type.units:
            assert vfr_type.to_unit([1], unit, unit)[0] == pytest.approx(1, rel=1e-5)
            ip_vals, ip_u = vfr_type.to_ip([1], unit)
            assert len(ip_vals) == 1
            si_vals, si_u = vfr_type.to_si([1], unit)
            assert len(si_vals) == 1
            for other_unit in vfr_type.units:
                assert len(vfr_type.to_unit([1], other_unit, unit)) == 1
        assert vfr_type.to_unit([1], 'ft3/s', 'm3/s')[0] == pytest.approx(35.3147, rel=1e-3)
        assert vfr_type.to_unit([1], 'L/s', 'm3/s')[0] == 1000
        assert vfr_type.to_unit([1], 'cfm', 'm3/s')[0] == pytest.approx(2118.88, rel=1e-1)
        assert vfr_type.to_unit([1], 'gpm', 'm3/s')[0] == pytest.approx(15850.3231, rel=1e-3)
        assert vfr_type.to_unit([1], 'mL/s', 'm3/s')[0] == 1000000
        assert vfr_type.to_unit([1], 'fl oz/s', 'm3/s')[0] == pytest.approx(33814, rel=1e-3)
        assert vfr_type.to_unit([1], 'm3/s', 'ft3/s')[0] == pytest.approx(1 / 35.3147, rel=1e-4)
        assert vfr_type.to_unit([1], 'm3/s', 'L/s')[0] == 0.001
        assert vfr_type.to_unit([1], 'm3/s', 'cfm')[0] == pytest.approx(1 / 2118.88, rel=1e-7)
        assert vfr_type.to_unit([1], 'm3/s', 'gpm')[0] == pytest.approx(1 / 15850.3231, rel=1e-8)
        assert vfr_type.to_unit([1], 'm3/s', 'mL/s')[0] == 0.000001
        assert vfr_type.to_unit([1], 'm3/s', 'fl oz/s')[0] == pytest.approx(1 / 33814, rel=1e-7)

    def test_mass_flow_rate(self):
        """Test MassFlowRate type."""
        mfr_type = massflowrate.MassFlowRate()
        for unit in mfr_type.units:
            assert mfr_type.to_unit([1], unit, unit)[0] == pytest.approx(1, rel=1e-5)
            ip_vals, ip_u = mfr_type.to_ip([1], unit)
            assert len(ip_vals) == 1
            si_vals, si_u = mfr_type.to_si([1], unit)
            assert len(si_vals) == 1
            for other_unit in mfr_type.units:
                assert len(mfr_type.to_unit([1], other_unit, unit)) == 1
        assert mfr_type.to_unit([1], 'lb/s', 'kg/s')[0] == pytest.approx(2.2046, rel=1e-2)
        assert mfr_type.to_unit([1], 'g/s', 'kg/s')[0] == 1000
        assert mfr_type.to_unit([1], 'oz/s', 'kg/s')[0] == pytest.approx(35.274, rel=1e-2)
        assert mfr_type.to_unit([1], 'kg/s', 'lb/s')[0] == pytest.approx(1 / 2.2046, rel=1e-4)
        assert mfr_type.to_unit([1], 'kg/s', 'g/s')[0] == 0.001
        assert mfr_type.to_unit([1], 'kg/s', 'oz/s')[0] == pytest.approx(1 / 35.274, rel=1e-5)

    def test_u_value(self):
        """Test UValue type."""
        uval_type = uvalue.UValue()
        for unit in uval_type.units:
            assert uval_type.to_unit([1], unit, unit)[0] == pytest.approx(1, rel=1e-5)
            ip_vals, ip_u = uval_type.to_ip([1], unit)
            assert len(ip_vals) == 1
            si_vals, si_u = uval_type.to_si([1], unit)
            assert len(si_vals) == 1
            for other_unit in uval_type.units:
                assert len(uval_type.to_unit([1], other_unit, unit)) == 1
        assert uval_type.to_unit([1], 'Btu/h-ft2-F', 'W/m2-K')[0] == pytest.approx(1 / 5.678263337, rel=1e-5)
        assert uval_type.to_unit([1], 'W/m2-K', 'Btu/h-ft2-F')[0] == pytest.approx(5.678263337, rel=1e-4)

    def test_r_value(self):
        """Test RValue type."""
        rval_type = rvalue.RValue()
        for unit in rval_type.units:
            assert rval_type.to_unit([1], unit, unit)[0] == pytest.approx(1, rel=1e-5)
            ip_vals, ip_u = rval_type.to_ip([1], unit)
            assert len(ip_vals) == 1
            si_vals, si_u = rval_type.to_si([1], unit)
            assert len(si_vals) == 1
            for other_unit in rval_type.units:
                assert len(rval_type.to_unit([1], other_unit, unit)) == 1
        assert rval_type.to_unit([1], 'h-ft2-F/Btu', 'm2-K/W')[0] == pytest.approx(5.678263337, rel=1e-4)
        assert rval_type.to_unit([1], 'clo', 'm2-K/W')[0] == pytest.approx(1 / 0.155, rel=1e-2)
        assert rval_type.to_unit([1], 'm2-K/W', 'h-ft2-F/Btu')[0] == pytest.approx(1 / 5.678263337, rel=1e-5)
        assert rval_type.to_unit([1], 'm2-K/W', 'clo')[0] == pytest.approx(0.155, rel=1e-3)

    def test_thermal_condition(self):
        """Test ThermalCondition type."""
        tc_type = thermalcondition.ThermalCondition()
        for unit in tc_type.units:
            assert tc_type.to_unit([1], unit, unit)[0] == pytest.approx(1, rel=1e-5)
            ip_vals, ip_u = tc_type.to_ip([1], unit)
            assert len(ip_vals) == 1
            si_vals, si_u = tc_type.to_si([1], unit)
            assert len(si_vals) == 1
            for other_unit in tc_type.units:
                assert len(tc_type.to_unit([1], other_unit, unit)) == 1
        assert tc_type.to_unit([1], 'PMV', 'condition')[0] == 1
        assert tc_type.to_unit([1], 'condition', 'PMV')[0] == 1

    def test_specific_energy(self):
        """Test ThermalCondition type."""
        tc_type = specificenergy.SpecificEnergy()
        for unit in tc_type.units:
            assert tc_type.to_unit([1], unit, unit)[0] == pytest.approx(1, rel=1e-5)
            ip_vals, ip_u = tc_type.to_ip([1], unit)
            assert len(ip_vals) == 1
            si_vals, si_u = tc_type.to_si([1], unit)
            assert len(si_vals) == 1
            for other_unit in tc_type.units:
                assert len(tc_type.to_unit([1], other_unit, unit)) == 1
        assert tc_type.to_unit([1], 'kBtu/lb', 'kWh/kg')[0] == pytest.approx(1.54772, rel=1e-4)
        assert tc_type.to_unit([1], 'Btu/lb', 'kWh/kg')[0] == pytest.approx(1547.72, rel=1e-2)
        assert tc_type.to_unit([1], 'Wh/kg', 'kWh/kg')[0] == pytest.approx(1000, rel=1e-1)
        assert tc_type.to_unit([1], 'J/kg', 'kWh/kg')[0] == pytest.approx(3600000, rel=1e-1)
        assert tc_type.to_unit([1], 'kJ/kg', 'kWh/kg')[0] == pytest.approx(3600, rel=1e-1)
        assert tc_type.to_unit([1], 'kWh/kg', 'kBtu/lb')[0] == pytest.approx(0.646111699, rel=1e-5)
        assert tc_type.to_unit([1], 'kWh/kg', 'Btu/lb')[0] == pytest.approx(0.00064611169979, rel=1e-7)
        assert tc_type.to_unit([1], 'kWh/kg', 'Wh/kg')[0] == pytest.approx(0.001, rel=1e-3)
        assert tc_type.to_unit([1], 'kWh/kg', 'J/kg')[0] == pytest.approx(2.7777777777777776e-07, rel=1e-9)
        assert tc_type.to_unit([1], 'kWh/kg', 'kJ/kg')[0] == pytest.approx(0.0002777777777777778, rel=1e-7)


if __name__ == "__main__":
    unittest.main()



# coding=utf-8
"""Ladybug color, colorsets and colorrange."""
from __future__ import division


class Color(object):
    """Ladybug RGB color.

    Attributes:
        r: red value 0-255, default: 0
        g: green value 0-255, default: 0
        b: blue red value 0-255, default: 0
    """

    def __init__(self, r, g, b):
        """Generate RGB Color.

        Args:
            r: red value 0-255, default: 0
            g: green value 0-255, default: 0
            b: blue red value 0-255, default: 0
        """
        self.r = r
        self.g = g
        self.b = b

    @property
    def r(self):
        """Return R value."""
        return self._r

    @r.setter
    def r(self, value):
        assert 0 <= int(value) <= 255, "%d is out of range. " % value + \
            "R value should be between 0-255"
        self._r = int(value)

    @property
    def g(self):
        """Return G value."""
        return self._g

    @g.setter
    def g(self, value):
        assert 0 <= int(value) <= 255, "%d is out of range. " % value + \
            "G value should be between 0-255"
        self._g = int(value)

    @property
    def b(self):
        """Return B value."""
        return self._b

    @b.setter
    def b(self, value):
        assert 0 <= int(value) <= 255, "%d is out of range. " % value + \
            "B value should be between 0-255"
        self._b = int(value)

    def ToString(self):
        """Overwrite .NET ToString."""
        return self.__repr__()

    def __repr__(self):
        """Return RGB values."""
        return "<R:%d, G:%d, B:%d>" % (self._r, self._g, self._b)


# TODO: Add support for CMYK
class Colorset(object):
    """Ladybug Color-range repository.

    A list of default Ladybug colorsets for color range:
        0 - original Ladybug
        1 - nuanced Ladybug
        2 - Multi-colored Ladybug
        3 - View Analysis 1
        4 - View Analysis 2 (Red,Green,Blue)
        5 - Sunlight Hours
        6 - ecotect
        7 - thermal Comfort Percentage
        8 - thermal Comfort Colors
        9 - thermal Comfort Colors (UTCI)
        10 - Hot Hours
        11 - Cold Hours
        12 - Shade Benefit/Harm
        13 - thermal Comfort Colors v2 (UTCI)
        14 - Shade Harm
        15 - Shade Benefit
        16 - Black to White
        17 - CFD Colors 1
        18 - CFD Colors 2
        19 - Energy Balance
        20 - THERM
        21 - Cloud Cover

    Usage:

        # initiare colorsets
        cs = Colorset()
        print(cs[0])
        >> [<R:75, G:107, B:169>, <R:115, G:147, B:202>, <R:170, G:200, B:247>,
            <R:193, G:213, B:208>, <R:245, G:239, B:103>, <R:252, G:230, B:74>,
            <R:239, G:156, B:21>, <R:234, G:123, B:0>, <R:234, G:74, B:0>,
            <R:234, G:38, B:0>]
    """
    _colors = {
        0: [(75, 107, 169), (115, 147, 202), (170, 200, 247), (193, 213, 208),
            (245, 239, 103), (252, 230, 74), (239, 156, 21), (234, 123, 0),
            (234, 74, 0), (234, 38, 0)],
        1: [(49, 54, 149), (69, 117, 180), (116, 173, 209), (171, 217, 233),
            (224, 243, 248), (255, 255, 191), (254, 224, 144), (253, 174, 97),
            (244, 109, 67), (215, 48, 39), (165, 0, 38)],
        2: [(4, 25, 145), (7, 48, 224), (7, 88, 255), (1, 232, 255),
            (97, 246, 156), (166, 249, 86), (254, 244, 1), (255, 121, 0),
            (239, 39, 0), (138, 17, 0)],
        3: [(255, 20, 147), (240, 47, 145), (203, 117, 139), (160, 196, 133),
            (132, 248, 129), (124, 253, 132), (96, 239, 160), (53, 217, 203),
            (15, 198, 240), (0, 191, 255)],
        4: [(0, 13, 255), (0, 41, 234), (0, 113, 181), (0, 194, 122),
            (0, 248, 82), (8, 247, 75), (64, 191, 58), (150, 105, 32),
            (225, 30, 9), (255, 0, 0)],
        5: [(55, 55, 55), (235, 235, 235)],
        6: [(0, 0, 255), (53, 0, 202), (107, 0, 148), (160, 0, 95), (214, 0, 41),
            (255, 12, 0), (255, 66, 0), (255, 119, 0), (255, 173, 0), (255, 226, 0),
            (255, 255, 0)],
        7: [(0, 0, 0), (110, 0, 153), (255, 0, 0), (255, 255, 102), (255, 255, 255)],
        8: [(0, 136, 255), (200, 225, 255), (255, 255, 255), (255, 230, 230),
            (255, 0, 0)],
        9: [(0, 136, 255), (67, 176, 255), (134, 215, 255), (174, 228, 255),
            (215, 242, 255), (255, 255, 255), (255, 243, 243), (255, 0, 0)],
        10: [(255, 255, 255), (255, 0, 0)],
        11: [(255, 255, 255), (0, 136, 255)],
        12: [(5, 48, 97), (33, 102, 172), (67, 147, 195), (146, 197, 222),
             (209, 229, 240), (255, 255, 255), (253, 219, 199), (244, 165, 130),
             (214, 96, 77), (178, 24, 43), (103, 0, 31)],
        13: [(5, 48, 97), (33, 102, 172), (67, 147, 195), (146, 197, 222),
             (209, 229, 240), (255, 255, 255), (244, 165, 130), (178, 24, 43)],
        14: [(255, 255, 255), (253, 219, 199), (244, 165, 130), (214, 96, 77),
             (178, 24, 43), (103, 0, 31)],
        15: [(255, 255, 255), (209, 229, 240), (146, 197, 222), (67, 147, 195),
             (33, 102, 172), (5, 48, 97)],
        16: [(0, 0, 0), (255, 255, 255)],
        17: [(0, 16, 120), (38, 70, 160), (5, 180, 222), (16, 180, 109),
             (59, 183, 35), (143, 209, 19), (228, 215, 29), (246, 147, 17),
             (243, 74, 0), (255, 0, 0)],
        18: [(69, 92, 166), (66, 128, 167), (62, 176, 168), (78, 181, 137),
             (120, 188, 59), (139, 184, 46), (197, 157, 54), (220, 144, 57),
             (228, 100, 59), (233, 68, 60)],
        19: [(138, 17, 0), (239, 39, 0), (255, 121, 0), (254, 244, 1),
             (166, 249, 86), (97, 246, 156), (1, 232, 255), (7, 88, 255),
             (4, 25, 145), (128, 102, 64)],
        20: [(0, 0, 0), (137, 0, 139), (218, 0, 218), (196, 0, 255), (0, 92, 255),
             (0, 198, 252), (0, 244, 215), (0, 220, 101), (7, 193, 0), (115, 220, 0),
             (249, 251, 0), (254, 178, 0), (253, 77, 0), (255, 15, 15),
             (255, 135, 135), (255, 255, 255)],
        21: [(0, 251, 255), (255, 255, 255), (217, 217, 217), (83, 114, 115)]
    }

    def __init__(self):
        """Initialize Color-sets."""
        pass

    @classmethod
    def original(cls):
        """original Ladybug colors."""
        return tuple(Color(*color) for color in cls._colors[0])

    @classmethod
    def nuanced(cls):
        """nuanced Ladybug colors."""
        return tuple(Color(*color) for color in cls._colors[1])

    @classmethod
    def multi_colored(cls):
        """Multi-colored legend."""
        return tuple(Color(*color) for color in cls._colors[2])

    @classmethod
    def view_analysis1(cls):
        """View analysis colors."""
        return tuple(Color(*color) for color in cls._colors[3])

    @classmethod
    def view_analysis2(cls):
        """View Analysis 2 colors."""
        return tuple(Color(*color) for color in cls._colors[4])

    @classmethod
    def sunlight_hours(cls):
        """sunlight_hours colors."""
        return tuple(Color(*color) for color in cls._colors[5])

    @classmethod
    def ecotect(cls):
        """ecotect colors."""
        return tuple(Color(*color) for color in cls._colors[6])

    @classmethod
    def thermal_comfort_percentage(cls):
        """thermal Comfort percentage."""
        return tuple(Color(*color) for color in cls._colors[7])

    @classmethod
    def thermal_comfort(cls):
        """thermal Comfort colors."""
        return tuple(Color(*color) for color in cls._colors[8])

    @classmethod
    def thermal_comfort_utci_1(cls):
        """thermal Comfort UTCI 1."""
        return tuple(Color(*color) for color in cls._colors[9])

    @classmethod
    def hot_hours(cls):
        """Hot Hours."""
        return tuple(Color(*color) for color in cls._colors[10])

    @classmethod
    def cold_hours(cls):
        """Cold Hours."""
        return tuple(Color(*color) for color in cls._colors[11])

    @classmethod
    def shade_benefit_harm(cls):
        """Shade Benefit Harm colors."""
        return tuple(Color(*color) for color in cls._colors[12])

    @classmethod
    def thermal_comfort_utci_2(cls):
        """thermal Comfort UTCI 2."""
        return tuple(Color(*color) for color in cls._colors[13])

    @classmethod
    def shade_harm(cls):
        """Shade Harm colors."""
        return tuple(Color(*color) for color in cls._colors[14])

    @classmethod
    def shade_benefit(cls):
        """Shade Benefit colors."""
        return tuple(Color(*color) for color in cls._colors[15])

    @classmethod
    def black_to_white(cls):
        """Black to white colors."""
        return tuple(Color(*color) for color in cls._colors[16])

    @classmethod
    def cfd_colors_1(cls):
        """CFD colors 1."""
        return tuple(Color(*color) for color in cls._colors[17])

    @classmethod
    def cfd_colors_2(cls):
        """CFD colors 2."""
        return tuple(Color(*color) for color in cls._colors[18])

    @classmethod
    def energy_balance(cls):
        """Energy Balance colors."""
        return tuple(Color(*color) for color in cls._colors[19])

    @classmethod
    def therm(cls):
        """THERM colors."""
        return tuple(Color(*color) for color in cls._colors[20])

    @classmethod
    def cloud_cover(cls):
        """Cloud Cover colors."""
        return tuple(Color(*color) for color in cls._colors[21])

    def __len__(self):
        """Return length of colors."""
        return len(self._colors)

    def __getitem__(self, key):
        """Return key item from the color list."""
        return [Color(*color) for color in self._colors[key]]

    def __setitem__(self, key, value):
        """Set a color to a new color in color list."""
        self._colors[key] = value

    def __delitem__(self, key):
        """Remove a color from the color list."""
        del self._colors[key]

    def __iter__(self):
        """Use colors to iterate."""
        return iter(self._colors)


class ColorRange(object):
    """Ladybug Color-range repository.

    A list of default Ladybug colorRanges

    Args:
        range:
        colors: A list of colors. Colors should be input as R, G, B values.
            Default: Colorset[1]
        domain: A list of numbers or strings. For numerical values it should be
            sorted from min to max. Default: ['min', 'max']
        chart_type: 0: continuous, 1: segmented, 2: ordinal. Default: 0
            In segmented and ordinal mode number of values should match number of colors
            Ordinal values can be strings and well as numericals
    Usage:
        ##
        colorRange = ColorRange(chart_type = 1)
        colorRange.domain = [100, 2000]
        colorRange.colors = [Color(75, 107, 169), Color(245, 239, 103),
            Color(234, 38, 0)]
        print(colorRange.color(99))
        print(colorRange.color(100))
        print(colorRange.color(2000))
        print(colorRange.color(2001))
        >> <R:75, G:107, B:169>
        >> <R:245, G:239, B:103>
        >> <R:245, G:239, B:103>
        >> <R:234, G:38, B:0>

        ##
        colorRange = ColorRange(chart_type = 1)
        colorRange.domain = [100, 2000]
        colorRange.colors = [Color(75, 107, 169), Color(245, 239, 103),
            Color(234, 38, 0)]
        colorRange.color(300)
        >> <R:245, G:239, B:103>

        ##
        colorRange = ColorRange(chart_type = 2)
        colorRange.domain = ["cold", "comfortable", "hot"]
        colorRange.colors = [Color(75, 107, 169), Color(245, 239, 103),
            Color(234, 38, 0)]
        colorRange.color("comfortable")
        >> <R:245, G:239, B:103>

    """

    # TODO: write a Color object
    def __init__(self, colors=None, domain=None, chart_type=0):
        """Initiate Ladybug color range."""
        self.ctype = chart_type
        self._is_domain_set = False
        self.colors = colors
        self.domain = domain

    @property
    def is_domain_set(self):
        """Return if Domain is set for this color-range."""
        return self._is_domain_set

    @property
    def domain(self):
        """Return domain."""
        return self._domain

    @domain.setter
    def domain(self, dom):
        # check and prepare domain
        if not dom:
            dom = [0, 1]

        if 'min' in dom or 'max' in dom:
            self._domain = dom
            self._is_domain_set = False
        else:
            assert hasattr(dom, "__iter__"), "Domain should be an iterable type."
            # if domain is numerical it should be sorted
            try:
                dom = sorted(map(float, dom))
            except ValueError:
                if self._ctype != 2:
                    print("Text domains can only be used in ordinal mode.\n" +
                          "Type is changed to ordinal.")
                    self.ctype == 2

            if self._ctype == 0:
                # continuous
                # if type is continuous domain can only be 2 values
                # or at least 1 value less than number of colors
                if len(dom) == 2:
                    # remap domain based on colors
                    _step = float(dom[1] - dom[0]) / (len(self._colors) - 1)
                    _n = dom[0]
                    dom = [_n + c * _step for c in range(len(self._colors))]

                assert len(self._colors) >= len(dom), \
                    "For continuous colors length of domain should be 2 or equal" \
                    " to number of colors"

            elif self._ctype == 1:
                # segmented
                # Number of colors should be at least one more than number
                # of domain Values
                assert len(self._colors) > len(dom), "Length of colors " + \
                    "should be more than domain values for segmented colors"

            self._domain = dom
            self._is_domain_set = True

    @property
    def colors(self):
        """Return list of colors."""
        return self._colors

    @colors.setter
    def colors(self, cols):
        if not cols:
            self._colors = Colorset()[1]
        else:
            assert hasattr(cols, "__iter__"), "Colors should be an iterable type"
            try:
                cols = [col if isinstance(col, Color) else Color(
                    col.R, col.G, col.B) for col in cols]
            except Exception:
                try:
                    cols = [Color(col.Red, col.Green, col.Blue) for col in cols]
                except Exception:
                    raise ValueError("%s is not a vlid color" % str(cols))
            self._colors = cols

    @property
    def ctype(self):
        """Chart type."""
        return self._ctype

    @ctype.setter
    def ctype(self, t):
        assert 0 <= int(t) <= 2, "Chart Type should be between 0-2\n" + \
            "0: continuous, 1: segmented, 2: ordinal"
        self._ctype = int(t)

    def color(self, value):
        """Return color for an input value."""
        assert self._is_domain_set, \
            "Domain is not set. Use self.domain to set the domain."

        if self._ctype == 2:
            # if ordinal map the value and color
            try:
                return self._colors[self._domain.index(value)]
            except ValueError:
                raise ValueError(
                    "%s is not a valid input for ordinal type.\n" % str(value) +
                    "List of valid values are %s" % ";".join(map(str, self._domain))
                )

        if value < self._domain[0]:
            return self._colors[0]
        if value > self._domain[-1]:
            return self._colors[-1]

        # find the index of the value in domain
        for count, d in enumerate(self._domain):
            if d <= value <= self._domain[count + 1]:
                if self._ctype == 0:
                    return self._cal_color(value, count)
                if self._ctype == 1:
                    return self._colors[count + 1]

    def _cal_color(self, value, color_index):
        """Blend between two colors based on input value."""
        range_min_p = self._domain[color_index]
        range_p = self._domain[color_index + 1] - range_min_p
        try:
            factor = (value - range_min_p) / range_p
        except ZeroDivisionError:
            factor = 0

        min_color = self.colors[color_index]
        max_color = self.colors[color_index + 1]
        red = round(factor * (max_color.r - min_color.r) + min_color.r)
        green = round(factor * (max_color.g - min_color.g) + min_color.g)
        blue = round(factor * (max_color.b - min_color.b) + min_color.b)

        return Color(red, green, blue)

    def __len__(self):
        """Return length of colors."""
        return len(self._colors)

    def __getitem__(self, key):
        """Return key item from the color list."""
        return self._colors[key]

    def __setitem__(self, key, value):
        """Set a color to a new color in color list."""
        self._colors[key] = value

    def __delitem__(self, key):
        """Remove a color from the color list."""
        del self._colors[key]

    def __iter__(self):
        """Use colors to iterate."""
        return iter(self._colors)



# coding=utf-8
"""Utility functions for list operations."""
from __future__ import division

import collections


def flatten(input_list):
    """Return a flattened genertor from an input list.

    Usage:

        input_list = [['a'], ['b', 'c', 'd'], [['e']], ['f']]
        list(flatten(input_list))
        >> ['a', 'b', 'c', 'd', 'e', 'f']
    """
    for el in input_list:
        if isinstance(el, collections.Iterable) \
                and not isinstance(el, basestring):
            for sub in flatten(el):
                yield sub
        else:
            yield el


def unflatten(guide, falttened_input):
    """Unflatten a falttened generator.

    Args:
        guide: A guide list to follow the structure
        falttened_input: A flattened iterator object

    Usage:

        guide = [["a"], ["b","c","d"], [["e"]], ["f"]]
        input_list = [0, 1, 2, 3, 4, 5, 6, 7]
        unflatten(guide, iter(input_list))
        >> [[0], [1, 2, 3], [[4]], [5]]
    """
    return [unflatten(sub_list, falttened_input) if isinstance(sub_list, list)
            else next(falttened_input) for sub_list in guide]


def duplicate(value, list_length):
    """Take a single value and duplicate it a certain number of times.

    Args:
        value: A value that you want to duplicate
        list_length: The number of times to duplicate the object.

    Usage:

        value = 1.2
        list_length = 5
        duplicate(value, list_length)
        >> [ 1.2, 1.2, 1.2, 1.2, 1.2]
    """
    return [value] * list_length



# coding=utf-8

import unittest
import pytest
import os
from ladybug.wea import Wea
from ladybug.location import Location
from ladybug.epw import EPW


class WeaTestCase(unittest.TestCase):
    """Test for (ladybug/epw.py)"""

    def test_from_file(self):
        """Test import from wea file."""
        wea_file = './tests/wea/san_francisco_10min.wea'
        with pytest.raises(Exception):
            Wea.from_file(wea_file)  # wrong timestep

        wea = Wea.from_file(wea_file, 6)
        assert wea.direct_normal_irradiance[45] == 88
        assert wea.diffuse_horizontal_irradiance[45] == 1
        assert wea.direct_normal_irradiance[46] == 313
        assert wea.diffuse_horizontal_irradiance[46] == 3

    def test_from_epw(self):
        """Test import from epw"""
        epw_path = './tests/epw/chicago.epw'
        wea_from_epw = Wea.from_epw_file(epw_path)

        assert wea_from_epw.location.city == 'Chicago Ohare Intl Ap'
        assert wea_from_epw.timestep == 1
        assert wea_from_epw.direct_normal_irradiance[7] == 22
        assert wea_from_epw.direct_normal_irradiance.datetimes[7].hour == 7
        assert wea_from_epw.direct_normal_irradiance.datetimes[7].minute == 0
        assert wea_from_epw.direct_normal_irradiance[8] == 397
        assert wea_from_epw.direct_normal_irradiance.datetimes[8].hour == 8
        assert wea_from_epw.direct_normal_irradiance.datetimes[8].minute == 0
        # diffuse horizontal irradiance
        assert wea_from_epw.diffuse_horizontal_irradiance[7] == 10
        assert wea_from_epw.diffuse_horizontal_irradiance.datetimes[7].hour == 7
        assert wea_from_epw.diffuse_horizontal_irradiance.datetimes[7].minute == 0
        assert wea_from_epw.diffuse_horizontal_irradiance[8] == 47
        assert wea_from_epw.diffuse_horizontal_irradiance.datetimes[8].hour == 8
        assert wea_from_epw.diffuse_horizontal_irradiance.datetimes[8].minute == 0

    def test_from_stat(self):
        """Test import from stat"""
        stat_path = './tests/stat/chicago.stat'
        wea_from_stat = Wea.from_stat_file(stat_path)

        assert wea_from_stat.location.city == 'Chicago Ohare Intl Ap'
        assert wea_from_stat.timestep == 1
        assert wea_from_stat.diffuse_horizontal_irradiance[0] == \
            pytest.approx(0, rel=1e-3)
        assert wea_from_stat.direct_normal_irradiance[0] == \
            pytest.approx(0, rel=1e-3)
        assert wea_from_stat.diffuse_horizontal_irradiance[12] == \
            pytest.approx(87.44171, rel=1e-3)
        assert wea_from_stat.direct_normal_irradiance[12] == \
            pytest.approx(810.693919, rel=1e-3)

    def test_from_stat_missing_optical(self):
        """Test import from a stat file that is missing optical data"""
        stat_path = './tests/stat/santamonica.stat'
        with pytest.raises(ValueError,
                           match='Stat file contains no optical data.'):
            Wea.from_stat_file(stat_path)

    def test_from_clear_sky(self):
        """Test from original clear sky"""
        location = Location(
            'Chicago Ohare Intl Ap', '-', 'USA', 41.98, -87.92, -6.0, 201.0)
        wea_from_clear_sky = Wea.from_ashrae_clear_sky(location)

        assert wea_from_clear_sky.location.city == 'Chicago Ohare Intl Ap'
        assert wea_from_clear_sky.timestep == 1
        assert wea_from_clear_sky.diffuse_horizontal_irradiance[0] == \
            pytest.approx(0, rel=1e-3)
        assert wea_from_clear_sky.direct_normal_irradiance[0] == \
            pytest.approx(0, rel=1e-3)
        assert wea_from_clear_sky.diffuse_horizontal_irradiance[12] == \
            pytest.approx(60.72258, rel=1e-3)
        assert wea_from_clear_sky.direct_normal_irradiance[12] == \
            pytest.approx(857.00439, rel=1e-3)

    def test_from_zhang_huang(self):
        """Test from zhang huang solar model"""
        path = './tests/epw/chicago.epw'
        epw = EPW(path)

        wea_from_zh = Wea.from_zhang_huang_solar(epw.location,
                                                 epw.total_sky_cover.values,
                                                 epw.relative_humidity.values,
                                                 epw.dry_bulb_temperature.values,
                                                 epw.wind_speed.values,
                                                 epw.atmospheric_station_pressure.values)

        assert wea_from_zh.location.city == 'Chicago Ohare Intl Ap'
        assert wea_from_zh.timestep == 1
        assert wea_from_zh.global_horizontal_irradiance[0] == \
            pytest.approx(0, rel=1e-1)
        assert wea_from_zh.global_horizontal_irradiance[12] == \
            pytest.approx(417.312, rel=1e-1)
        assert wea_from_zh.direct_normal_irradiance[12] == \
            pytest.approx(654.52, rel=1e-1)
        assert wea_from_zh.diffuse_horizontal_irradiance[12] == \
            pytest.approx(144.51, rel=1e-1)

    def test_zhang_huang_accuracy(self):
        """Test zhang huang solar model to ensure that average error is within
        25% of actual solar."""
        path = './tests/epw/chicago.epw'
        epw = EPW(path)

        wea = Wea.from_zhang_huang_solar(epw.location,
                                         epw.total_sky_cover.values,
                                         epw.relative_humidity.values,
                                         epw.dry_bulb_temperature.values,
                                         epw.wind_speed.values,
                                         epw.atmospheric_station_pressure.values)
        # test global horizontal radiation
        glob_horiz_error = [abs(i - j) for i, j in zip(
            epw.global_horizontal_radiation,
            wea.global_horizontal_irradiance)]
        avg_glob_horiz_error = sum(glob_horiz_error) / sum(
            epw.global_horizontal_radiation)
        assert (sum(glob_horiz_error) / 8760) < 50
        assert avg_glob_horiz_error < 0.25

        # test direct normal radiation
        dir_normal_error = [abs(i - j) for i, j in zip(
            epw.direct_normal_radiation, wea.direct_normal_irradiance)]
        avg_dir_normal_error = sum(dir_normal_error) / sum(
            epw.direct_normal_radiation)
        assert sum(dir_normal_error) / 8760 < 100
        assert avg_dir_normal_error < 0.5

        # test diffuse horizontal radiation
        dif_horiz_error = [abs(i - j) for i, j in zip(
            epw.diffuse_horizontal_radiation,
            wea.diffuse_horizontal_irradiance)]
        avg_dif_horiz_error = sum(dif_horiz_error) / sum(
            epw.diffuse_horizontal_radiation)
        assert sum(dif_horiz_error) / 8760 < 50
        assert avg_dif_horiz_error < 0.5

    def test_json_methods(self):
        """Test JSON serialization methods"""
        epw_path = './tests/epw/chicago.epw'
        wea = Wea.from_epw_file(epw_path)

        assert wea.to_json() == Wea.from_json(wea.to_json()).to_json()

    def test_import_epw(self):
        """Test to compare import from epw with its json version."""
        epw_path = './tests/epw/chicago.epw'

        wea_from_epw = Wea.from_epw_file(epw_path)

        wea_json = wea_from_epw.to_json()
        wea_from_json = Wea.from_json(wea_json)
        assert wea_from_json.direct_normal_irradiance.values == \
            wea_from_epw.direct_normal_irradiance.values
        assert wea_from_json.diffuse_horizontal_irradiance.values == \
            wea_from_epw.diffuse_horizontal_irradiance.values

    def test_import_stat(self):
        """Test to compare import from stat with its json version."""
        stat_path = './tests/stat/chicago.stat'
        wea_from_stat = Wea.from_stat_file(stat_path)

        wea_json = wea_from_stat.to_json()
        wea_from_json = Wea.from_json(wea_json)
        assert wea_from_json.direct_normal_irradiance.values == \
            wea_from_stat.direct_normal_irradiance.values
        assert wea_from_json.diffuse_horizontal_irradiance.values == \
            wea_from_stat.diffuse_horizontal_irradiance.values

    def test_write_wea(self):
        """Test the write Wea file capability."""
        stat_path = './tests/stat/chicago.stat'
        wea_from_stat = Wea.from_stat_file(stat_path)

        wea_path = './tests/wea/chicago_stat.wea'
        hrs_path = './tests/wea/chicago_stat.hrs'
        hoys = range(8760)
        wea_from_stat.write(wea_path, hoys, True)

        assert os.path.isfile(wea_path)
        assert os.stat(wea_path).st_size > 1
        assert os.path.isfile(hrs_path)
        assert os.stat(hrs_path).st_size > 1

        # check the order of the data in the file
        with open(wea_path) as wea_f:
            lines = wea_f.readlines()
            assert float(lines[6].split(' ')[-2]) == \
                pytest.approx(
                    wea_from_stat.direct_normal_irradiance[0], rel=1e-1)
            assert int(lines[6].split(' ')[-1]) == \
                wea_from_stat.diffuse_horizontal_irradiance[0]
            assert float(lines[17].split(' ')[-2]) == \
                pytest.approx(
                    wea_from_stat.direct_normal_irradiance[11], rel=1e-1)
            assert float(lines[17].split(' ')[-1]) == \
                pytest.approx(
                    wea_from_stat.diffuse_horizontal_irradiance[11], rel=1e-1)

        os.remove(wea_path)
        os.remove(hrs_path)

    def test_global_and_direct_horizontal(self):
        """Test the global horizontal irradiance on method."""
        stat_path = './tests/stat/chicago.stat'
        wea_from_stat = Wea.from_stat_file(stat_path)

        diffuse_horiz_rad = wea_from_stat.diffuse_horizontal_irradiance
        direct_horiz_rad = wea_from_stat.direct_horizontal_irradiance
        glob_horiz_rad = wea_from_stat.global_horizontal_irradiance

        assert [x for x in glob_horiz_rad] == pytest.approx(
            [x + y for x, y in zip(diffuse_horiz_rad, direct_horiz_rad)], rel=1e-3)

    def test_directional_irradiance(self):
        """Test the directinal irradiance method."""
        stat_path = './tests/stat/chicago.stat'
        wea_from_stat = Wea.from_stat_file(stat_path)

        srf_total, srf_direct, srf_diffuse, srf_reflect = \
            wea_from_stat.directional_irradiance(90)
        diffuse_horiz_rad = wea_from_stat.diffuse_horizontal_irradiance
        direct_horiz_rad = wea_from_stat.direct_horizontal_irradiance
        glob_horiz_rad = wea_from_stat.global_horizontal_irradiance

        assert srf_total.values == pytest.approx(glob_horiz_rad.values, rel=1e-3)
        assert srf_direct.values == pytest.approx(direct_horiz_rad.values, rel=1e-3)
        assert srf_diffuse.values == pytest.approx(diffuse_horiz_rad.values, rel=1e-3)
        assert srf_reflect.values == pytest.approx([0] * 8760, rel=1e-3)

    def test_leap_year(self):
        """Test clear sky with leap year."""
        location = Location(
            'Chicago Ohare Intl Ap', '-', 'USA', 41.98, -87.92, -6.0, 201.0)
        wea = Wea.from_ashrae_clear_sky(location, is_leap_year=True)

        assert wea.diffuse_horizontal_irradiance.datetimes[1416].month == 2
        assert wea.diffuse_horizontal_irradiance.datetimes[1416].day == 29
        assert wea.diffuse_horizontal_irradiance.datetimes[1416].hour == 0

        assert wea.diffuse_horizontal_irradiance.datetimes[1416 + 12].month == 2
        assert wea.diffuse_horizontal_irradiance.datetimes[1416 + 12].day == 29
        assert wea.diffuse_horizontal_irradiance.datetimes[1416 + 12].hour == 12


if __name__ == "__main__":
    unittest.main()



# coding=utf-8
"""Utility functions for working with files and directories."""
from __future__ import division

import os
import shutil
import zipfile
import sys

if (sys.version_info < (3, 0)):
    import urllib2
    readmode = 'rb'
    writemode = 'wb'
else:
    import urllib.request
    readmode = 'r'
    writemode = 'w'


def preparedir(target_dir, remove_content=True):
    """Prepare a folder for analysis.

    This method creates the folder if it is not created, and removes the file in
    the folder if the folder already existed.
    """
    if os.path.isdir(target_dir):
        if remove_content:
            nukedir(target_dir, False)
        return True
    else:
        try:
            os.makedirs(target_dir)
            return True
        except Exception as e:
            print("Failed to create folder: %s\n%s" % (target_dir, e))
            return False


def nukedir(target_dir, rmdir=False):
    """Delete all the files inside target_dir.

    Usage:
        nukedir("c:/ladybug/libs", True)
    """
    d = os.path.normpath(target_dir)

    if not os.path.isdir(d):
        return

    files = os.listdir(d)

    for f in files:
        if f == '.' or f == '..':
            continue
        path = os.path.join(d, f)

        if os.path.isdir(path):
            nukedir(path)
        else:
            try:
                os.remove(path)
            except Exception:
                print("Failed to remove %s" % path)

    if rmdir:
        try:
            os.rmdir(d)
        except Exception:
            print("Failed to remove %s" % d)


def write_to_file_by_name(folder, fname, data, mkdir=False):
    """Write a string of data to file by filename and folder.

    Args:
        folder: Target folder (e.g. c:/ladybug).
        fname: File name (e.g. testPts.pts).
        data: Any data as string.
        mkdir: Set to True to create the directory if doesn't exist (Default: False).
    """
    if not os.path.isdir(folder):
        if mkdir:
            preparedir(folder)
        else:
            created = preparedir(folder, False)
            if not created:
                raise ValueError("Failed to find %s." % folder)

    file_path = os.path.join(folder, fname)

    with open(file_path, writemode) as outf:
        try:
            outf.write(str(data))
            return file_path
        except Exception as e:
            raise IOError("Failed to write %s to file:\n\t%s" % (fname, str(e)))


def write_to_file(file_path, data, mkdir=False):
    """Write a string of data to file.

    Args:
        file_path: Full path for a valid file path (e.g. c:/ladybug/testPts.pts)
        data: Any data as string
        mkdir: Set to True to create the directory if doesn't exist (Default: False)
    """
    folder, fname = os.path.split(file_path)
    return write_to_file_by_name(folder, fname, data, mkdir)


def copy_files_to_folder(files, target_folder, overwrite=True):
    """Copy a list of files to a new target folder.

    Returns:
        A list of fullpath of the new files.
    """
    if not files:
        return []

    for f in files:
        target = os.path.join(target_folder, os.path.split(f)[-1])

        if target == f:
            # both file path are the same!
            return target

        if os.path.exists(target):
            if overwrite:
                # remove the file before copying
                try:
                    os.remove(target)
                except Exception:
                    raise IOError("Failed to remove %s" % f)
                else:
                    shutil.copy(f, target)
            else:
                continue
        else:
            print('Copying %s to %s' % (os.path.split(f)[-1],
                                        os.path.normpath(target_folder)))
            shutil.copy(f, target)

    return [os.path.join(target_folder, os.path.split(f)[-1]) for f in files]


def bat_to_sh(file_path):
    """Convert honeybee .bat file to .sh file.

    WARNING: This is a very simple function and doesn't handle any edge cases.
    """
    sh_file = file_path[:-4] + '.sh'
    with open(file_path, 'rb') as inf, open(sh_file, 'wb') as outf:
        outf.write('#!/usr/bin/env bash\n\n')
        for line in inf:
            # pass the path lines, etc to get to the commands
            if line.strip():
                continue
            else:
                break

        for line in inf:
            if line.startswith('echo'):
                continue
            modified_line = line.replace('c:\\radiance\\bin\\', '').replace('\\', '/')
            outf.write(modified_line)

    print('bash file is created at:\n\t%s' % sh_file)
    # Heroku - Make command.sh executable
    st = os.stat(sh_file)
    os.chmod(sh_file, st.st_mode | 0o111)
    return sh_file


def _download_py2(link, path, __hdr__):
    """Download a file from a link in Python 2."""
    try:
        req = urllib2.Request(link, headers=__hdr__)
        u = urllib2.urlopen(req)
    except Exception as e:
        raise Exception(' Download failed with the error:\n{}'.format(e))

    with open(path, 'wb') as outf:
        for l in u:
            outf.write(l)
    u.close()


def _download_py3(link, path, __hdr__):
    """Download a file from a link in Python 3."""
    try:
        req = urllib.request.Request(link, headers=__hdr__)
        u = urllib.request.urlopen(req)
    except Exception as e:
        raise Exception(' Download failed with the error:\n{}'.format(e))

    with open(path, 'wb') as outf:
        for l in u:
            outf.write(l)
    u.close()


def download_file_by_name(url, target_folder, file_name, mkdir=False):
    """Download a file to a directory.

    Args:
        url: A string to a valid URL.
        target_folder: Target folder for download (e.g. c:/ladybug)
        file_name: File name (e.g. testPts.zip).
        mkdir: Set to True to create the directory if doesn't exist (Default: False)
    """
    # headers to "spoof" the download as coming from a browser (needed for E+ site)
    __hdr__ = {'User-Agent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.11 '
               '(KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11',
               'Accept': 'text/html,application/xhtml+xml,'
               'application/xml;q=0.9,*/*;q=0.8',
               'Accept-Charset': 'ISO-8859-1,utf-8;q=0.7,*;q=0.3',
               'Accept-Encoding': 'none',
               'Accept-Language': 'en-US,en;q=0.8',
               'Connection': 'keep-alive'}

    # create the target directory.
    if not os.path.isdir(target_folder):
        if mkdir:
            preparedir(target_folder)
        else:
            created = preparedir(target_folder, False)
            if not created:
                raise ValueError("Failed to find %s." % target_folder)
    file_path = os.path.join(target_folder, file_name)

    if (sys.version_info < (3, 0)):
        _download_py2(url, file_path, __hdr__)
    else:
        _download_py3(url, file_path, __hdr__)


def download_file(url, file_path, mkdir=False):
    """Write a string of data to file.

    Args:
        url: A string to a valid URL.
        file_path: Full path to intended download location (e.g. c:/ladybug/testPts.pts)
        mkdir: Set to True to create the directory if doesn't exist (Default: False)
    """
    folder, fname = os.path.split(file_path)
    return download_file_by_name(url, folder, fname, mkdir)


def unzip_file(source_file, dest_dir=None, mkdir=False):
    """Unzip a compressed file.

    Args:
        source_file: Full path to a valid compressed file (e.g. c:/ladybug/testPts.zip)
        dest_dir: Target folder to extract to (e.g. c:/ladybug).
            Default is set to the same directory as the source file.
        mkdir: Set to True to create the directory if doesn't exist (Default: False)
    """
    # set default dest_dir and create it if need be.
    if dest_dir is None:
        dest_dir, fname = os.path.split(source_file)
    elif not os.path.isdir(dest_dir):
        if mkdir:
            preparedir(dest_dir)
        else:
            created = preparedir(dest_dir, False)
            if not created:
                raise ValueError("Failed to find %s." % dest_dir)

    # extract files to destination
    with zipfile.ZipFile(source_file) as zf:
        for member in zf.infolist():
            words = member.filename.split('\\')
            for word in words[:-1]:
                drive, word = os.path.splitdrive(word)
                head, word = os.path.split(word)
                if word in (os.curdir, os.pardir, ''):
                    continue
                dest_dir = os.path.join(dest_dir, word)
            zf.extract(member, dest_dir)


def csv_to_matrix(csv_file_path):
    """Load a CSV file into a Python matrix of strings.

    Args:
        csv_file_path: Full path to a valid CSV file (e.g. c:/ladybug/test.csv)
    """
    mtx = []
    with open(csv_file_path) as csv_data_file:
        for row in csv_data_file:
            mtx.append(row.split(','))
    return mtx


def csv_to_num_matrix(csv_file_path):
    """Load a CSV file consisting only of numbers into a Python matrix of floats.

    Args:
        csv_file_path: Full path to a valid CSV file (e.g. c:/ladybug/test.csv)
    """
    mtx = []
    with open(csv_file_path) as csv_data_file:
        for row in csv_data_file:
            mtx.append([float(val) for val in row.split(',')])
    return mtx



# These must be imported here in this order because of how they reference eachother.
import ladybug._datacollectionbase
import ladybug.datacollection
import ladybug.datacollectionimmutable

import sys
import importlib
import pkgutil


#  find and import ladybug plugins
#  this is a critical step to add additional functionalities to ladybug core library.
lb_plugins = {
    name: importlib.import_module(name)
    for finder, name, ispkg
    in pkgutil.iter_modules()
    if name.startswith('ladybug_')
}

for key in lb_plugins:
    print('Successfully imported Ladybug plugin: {}'.format(key))

# This is a variable to check if the library is a [+] library.
setattr(sys.modules[__name__], 'isplus', False)



# coding=utf-8
from __future__ import division

from color import ColorRange
from listoperations import flatten, unflatten


class LegendParameters(object):
    """Ladybug lagend parameters.

    Attributes:
        legend_range: Input a list of numbers or strings to set the boundary of
            legend. The default is ['min', 'max']
        number_of_segments: An interger representing the number of steps between
            the high and low boundary of the legend. The default is set to 11
            and any custom values put in here should always be greater than or
            equal to 2.
        colors: An optional list of colors. Default is Ladybug's default colorset
        chart_type: 0: continuous, 1: segmented, 2: ordinal. Default: 0.
            Ordinal values can be strings and well as numericals
        title:  Legend title. It's usually analysis unit
        font: An optional text string that sets the font of the text. Examples
            include "Arial", "Times New Roman" or "Courier" (all without
            quotations). The text input here can be any font that is on your
            computer but the font must be of an Editable file type (as seen in
            the font folder off of your control panel).  Font files that are
            print and Preview will not work.  If you wish to use a Bold version
            of the font, include a ", Bold" at the end of the font name
            (example: "Arial, Bold").
        fontSize: An optional number to set the size of the text in model's units.
        scale: Input a number here to change the scale of the legend.
            The default is set to 1.
        basePlane: Input a plane to change the location and orientation of the
            legend. The default is set to the right of the analysis scene in
            XY plane.
        vertical: Set to False to get a horizontal legend. Default is vertical.

    Usage:

        lp = LegendParameters(legend_range = [2, 28])
        print(lp.color(10))

    """

    _cType = {0: 'continuous', 1: 'segmented', 2: 'ordinal'}

    # TODO: Add textual and geometry parts
    def __init__(self, legend_range=None, number_of_segments=11,
                 colors=None, chart_type=0):
        """Init the class."""
        legend_range = legend_range or ['min', 'max']
        self.number_of_segments = number_of_segments or 11
        self.colorRange = ColorRange(colors=colors, domain=legend_range,
                                     chart_type=chart_type)

    @property
    def colors(self):
        """Get or set list of colors."""
        return self.colorRange.colors

    @colors.setter
    def colors(self, cols):
        self.colorRange.colors = cols

    @property
    def domain(self):
        """Get or set color range domain."""
        return self.colorRange.domain

    # TODO: set the domain based on the number of segments
    @domain.setter
    def domain(self, dom):
        self.colorRange.domain = dom

    @property
    def is_domain_set(self):
        """Check if the domain is set."""
        return self.colorRange.is_domain_set

    def set_domain(self, values):
        """Set domain of the colors based on min and max of a list of values."""
        _flattenedList = sorted(flatten(values))
        self.domain = tuple(_flattenedList[0] if d == 'min' else d for d in self.domain)
        self.domain = tuple(_flattenedList[-1] if d == 'max' else d for d in self.domain)

    def calculate_colors(self, values):
        """Return a list (or list of lists) of colors based on input values."""
        # set domain if it is not set
        _flattenedList = list(flatten(values))
        if not self.is_domain_set:
            self.set_domain(_flattenedList)

        _flattenedColors = range(len(_flattenedList))
        for count, value in enumerate(_flattenedList):
            _flattenedColors[count] = self.calculate_color(value)

        return unflatten(values, iter(_flattenedColors))

    def calculate_color(self, value):
        """Calculate color for a specific value."""
        return self.colorRange.color(value)

    def geometry(self):
        """Calculate legend geometry."""
        raise NotImplementedError()

    def text(self):
        """Return legend text."""
        raise NotImplementedError()

    def ToString(self):
        """Overwrite .NET ToString."""
        return self.__repr__()

    def __repr__(self):
        """Legend representation."""
        return "{} legend parameters".format(self._cType[self.colorRange.ctype])
